"""add ReleaseMonitor.airdate_stamp

Revision ID: 1d11afd9a90
Revises: 2ee6e71eb2f
Create Date: 2014-12-18 11:43:35.199129

"""

# revision identifiers, used by Alembic.
revision = '1d11afd9a90'
down_revision = '2ee6e71eb2f'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('release', sa.Column('airdate_stamp', sa.Integer(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    pass
